#pragma once
#ifndef PLAYER_ORBIT_H
#define PLAYER_ORBIT_H

#include "sgg/graphics.h"
#include <cmath>
#define M_PI 3.14159265358979323846

/**
 * Structure that defines the properties of a player, represented as a circle.
 */
struct Player {
    float x, y;       ///< Center position of the player.
    float radius;     ///< Radius of the circle representing the player.
    float speed;      ///< Speed of the player's movement.
    float angle;      ///< Angle in degrees for directional movements like inclined throws.
    float time;       ///< Time variable for time-based movements (e.g., free fall).
};

/**
 * Moves the player horizontally by a specified speed.
 * @param player Reference to the Player structure to be moved.
 * @param speed Horizontal speed to be applied to the player's movement.
 */
void moveHorizontal(Player& player, float speed) {
    player.x += speed;
}

/**
 * Moves the player vertically by a specified speed.
 * @param player Reference to the Player structure to be moved.
 * @param speed Vertical speed to be applied to the player's movement.
 */
void moveVertical(Player& player, float speed) {
    player.y += speed;
}

/**
 * Moves the player in a circular motion around a specified center.
 * @param player Reference to the Player structure to be moved.
 * @param centerX X-coordinate of the center of the circular path.
 * @param centerY Y-coordinate of the center of the circular path.
 * @param radius Radius of the circular path.
 * @param speed Angular speed of the circular motion in degrees per update.
 */
void moveCircular(Player& player, float centerX, float centerY, float radius, float speed) {
    player.angle += speed;
    player.x = centerX + radius * cos(player.angle * M_PI / 180.0f);
    player.y = centerY + radius * sin(player.angle * M_PI / 180.0f);
}

/**
 * Simulates free fall motion for the player under gravity.
 * @param player Reference to the Player structure to be moved.
 * @param gravity Acceleration due to gravity (default is 9.8 m/s^2).
 */
void freeFall(Player& player, float gravity = 9.8f) {
    player.y += 0.5f * gravity * player.time * player.time;
    player.time += 0.1f; // Updates time in each frame
}

/**
 * Simulates a vertical throw upwards for the player.
 * @param player Reference to the Player structure to be moved.
 * @param initialVelocity Initial vertical velocity of the throw.
 * @param gravity Acceleration due to gravity (default is 9.8 m/s^2).
 */
void verticalThrow(Player& player, float initialVelocity, float gravity = 9.8f) {
    player.y -= initialVelocity * player.time - 0.5f * gravity * player.time * player.time;
    player.time += 0.1f;
}

/**
 * Simulates an inclined throw at an angle for the player.
 * @param player Reference to the Player structure to be moved.
 * @param initialVelocity Initial velocity of the throw.
 * @param angle Angle of the throw in degrees.
 * @param gravity Acceleration due to gravity (default is 9.8 m/s^2).
 */
void inclinedThrow(Player& player, float initialVelocity, float angle, float gravity = 9.8f) {
    float radAngle = angle * M_PI / 180.0f;
    player.x += initialVelocity * cos(radAngle) * player.time;
    player.y -= (initialVelocity * sin(radAngle) * player.time - 0.5f * gravity * player.time * player.time);
    player.time += 0.1f;
}

/**
 * Moves the player in an elliptical path with a constant speed.
 * @param player Reference to the Player structure to be moved.
 * @param centerX X-coordinate of the center of the elliptical path.
 * @param centerY Y-coordinate of the center of the elliptical path.
 * @param a Semi-major axis length of the ellipse.
 * @param b Semi-minor axis length of the ellipse.
 * @param speed Angular speed of the elliptical motion in degrees per update.
 */
void ellipticalMotion(Player& player, float centerX, float centerY, float a, float b, float speed) {
    player.angle += speed;
    player.x = centerX + a * cos(player.angle * M_PI / 180.0f);
    player.y = centerY + b * sin(player.angle * M_PI / 180.0f);
}

/**
 * Draws the player as a circle at the current position.
 * @param player Constant reference to the Player structure to be drawn.
 */
void drawPlayer(const Player& player) {
    graphics::Brush brush;
    brush.fill_color[0] = 0.0f; // Blue color for the player
    brush.fill_color[1] = 0.0f;
    brush.fill_color[2] = 1.0f;
    graphics::drawDisk(player.x, player.y, player.radius, brush);
}

/**
 * Resets the time variable of the player for movement calculations.
 * @param player Reference to the Player structure whose time is to be reset.
 */
void resetTime(Player& player) {
    player.time = 0.0f;
}

#endif // PLAYER_ORBIT_H

